@@ -1,11 +1,11 @@
-\set MYSQL_HOST			'\'localhost\''
-\set MYSQL_PORT			'\'3306\''
-\set MYSQL_USER_NAME	'\'edb\''
-\set MYSQL_PASS			'\'edb\''
+\set MYSQL_HOST			`echo \'"$MYSQL_HOST"\'`
+\set MYSQL_PORT			`echo \'"$MYSQL_PORT"\'`
+\set MYSQL_USER_NAME	`echo \'"$MYSQL_USER_NAME"\'`
+\set MYSQL_PASS			`echo \'"$MYSQL_PWD"\'`
 
 -- Before running this file User must create database mysql_fdw_regress on
--- MySQL with all permission for 'edb' user with 'edb' password and ran
--- mysql_init.sh file to create tables.
+-- MySQL with all permission for MYSQL_USER_NAME user with MYSQL_PWD password
+-- and ran mysql_init.sh file to create tables.
 
 \c contrib_regression
 CREATE EXTENSION IF NOT EXISTS mysql_fdw;
@@ -29,6 +29,10 @@ CREATE FOREIGN TABLE f_test_tbl2 (c1 INTEGER, c2 VARCHAR(14), c3 VARCHAR(13))
 CREATE TYPE size_t AS enum('small','medium','large');
 CREATE FOREIGN TABLE f_enum_t1(id int, size size_t)
   SERVER mysql_svr OPTIONS (dbname 'mysql_fdw_regress', table_name 'enum_t1');
+CREATE FOREIGN TABLE test5_1(c1 INT, c2 CHAR, c3 VARCHAR, c4 BOOLEAN, c5 TEXT, c6 INTERVAL, c7 BYTEA, c8 pg_catalog.DATE, c9 NUMERIC, c10 NAME)
+  SERVER mysql_svr OPTIONS (dbname 'mysql_fdw_regress', table_name 'test5');
+CREATE FOREIGN TABLE test5_2(c1 INT, c2 BYTEA, c3 BYTEA, c4 BYTEA, c5 BYTEA, c6 BYTEA, c7 BYTEA, c8 BYTEA, c9 BYTEA, c10 BYTEA)
+  SERVER mysql_svr OPTIONS (dbname 'mysql_fdw_regress', table_name 'test5');
 
 -- Insert data in MySQL db using foreign tables
 INSERT INTO f_test_tbl1 VALUES (100, 'EMP1', 'ADMIN', 1300, '1980-12-17', 800.23, NULL, 20);
@@ -315,6 +319,25 @@ SELECT * FROM enum_t2 ORDER BY id;
 DROP FOREIGN TABLE enum_t1;
 DROP FOREIGN TABLE enum_t2;
 
+-- FDW-248: IMPORT FOREIGN SCHEMA command should work correctly if called
+-- multiple times. Earlier we wrongly used PG_TRY/CATCH block to get the server
+-- options without clearing the error state and that exceeded
+-- ERRORDATA_STACK_SIZE hard coded to 5.
+DO
+$DO$
+DECLARE
+  i int;
+BEGIN
+  FOR i IN 1..5
+  LOOP
+    IMPORT FOREIGN SCHEMA mysql_fdw_regress LIMIT TO (mysql_test)
+    FROM SERVER mysql_svr INTO public;
+
+    DROP FOREIGN TABLE mysql_test;
+  END LOOP;
+END;
+$DO$;
+
 -- Parameterized queries should work correctly.
 EXPLAIN (VERBOSE, COSTS OFF)
 SELECT c1, c2 FROM f_test_tbl1
@@ -356,6 +379,46 @@ SELECT t1.c1, (SELECT c2 FROM f_test_tbl1 WHERE c1 =(SELECT 500))
     SELECT c1, c2 FROM f_test_tbl2 WHERE c1 > ANY (SELECT 20)) t2
   ORDER BY 1, 2;
 
+-- FDW-255: Should throw an error when we select system attribute.
+SELECT xmin FROM f_test_tbl1;
+SELECT ctid, xmax, tableoid FROM f_test_tbl1;
+SELECT xmax, c1 FROM f_test_tbl1;
+SELECT count(tableoid) FROM f_test_tbl1;
+
+-- FDW-333: MySQL BINARY and VARBINARY data type should map to BYTEA in
+-- Postgres while importing the schema.
+IMPORT FOREIGN SCHEMA mysql_fdw_regress LIMIT TO ("test5")
+  FROM SERVER mysql_svr INTO public;
+SELECT attrelid::regclass, atttypid::regtype FROM pg_attribute
+  WHERE attrelid = 'test5'::regclass AND attnum > 1 ORDER BY 1;
+SELECT * FROM test5 ORDER BY 1;
+-- Test Mapping of MySQL BINARY and VARBINARY data type with various
+-- Postgres data types.
+SELECT * FROM test5_1 ORDER BY 1;
+SELECT * FROM test5_1 WHERE c9 IS NULL ORDER BY 1;
+SELECT * FROM test5_1 WHERE c10 IS NULL ORDER BY 1;
+-- Test MYSQL BINARY(n) and VARBINARY(n) variants mapping to Postgres BYTEA.
+SELECT * FROM test5_2 ORDER BY 1;
+SELECT * FROM test5_2 WHERE c9 IS NULL ORDER BY 1;
+SELECT * FROM test5_2 WHERE c10 IS NULL ORDER BY 1;
+
+-- FDW-400: Test the parameterized query by enabling use_remote_estimate
+-- option.
+ALTER SERVER mysql_svr options (SET use_remote_estimate 'true');
+SELECT c1, sum(c7) FROM f_test_tbl1 t1
+  GROUP BY c1 HAVING EXISTS
+    (SELECT 1 FROM f_test_tbl1 t2 WHERE (t1.c1 = t2.c1))
+  ORDER BY 1,2;
+ALTER SERVER mysql_svr options (SET use_remote_estimate 'false');
+
+-- FDW-411: Volatile/immutable functions should not get pushed down to remote
+-- MySQL server.
+EXPLAIN (VERBOSE, COSTS OFF)
+SELECT c1, c2, c3 FROM f_test_tbl1 WHERE pg_catalog.timeofday() IS NOT NULL
+  ORDER BY 1 limit 5;
+SELECT c1, c2, c3 FROM f_test_tbl1 WHERE pg_catalog.timeofday() IS NOT NULL
+  ORDER BY 1 limit 5;
+
 -- Cleanup
 DROP TABLE l_test_tbl1;
 DROP TABLE l_test_tbl2;
@@ -374,6 +437,9 @@ DROP FOREIGN TABLE f_numbers;
 DROP FOREIGN TABLE f_mysql_test;
 DROP FOREIGN TABLE f_enum_t1;
 DROP FOREIGN TABLE f_test_tbl3;
+DROP FOREIGN TABLE test5;
+DROP FOREIGN TABLE test5_1;
+DROP FOREIGN TABLE test5_2;
 DROP TYPE size_t;
 DROP TYPE enum_t1_size_t;
 DROP TYPE enum_t2_size_t;
